----------------------------------------------------------------------------------------
-- This code implements a decoder hardware from binary to hexadecimal for a 7-seg display
--
-- Elements of Digital Logic (SCC - 0512) - Institute of Mathematical and Computer Sciences
-- University of Sao Paulo
--
-- Author: Bruno de Abreu Silva
-- e-mail: brunoas@icmc.usp.br
-- Date: 2012/04/09
----------------------------------------------------------------------------------------

library ieee ;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity hexa7seg is
    port(
        bin: in std_logic_vector (0 to 3); -- bit 3 is the least significant bit
        seg: out std_logic_vector (0 to 6) 
    );
end hexa7seg;

architecture logicFunc of hexa7seg is
begin
    seg(0) <= (not(bin(0)) and not(bin(1)) and not(bin(2)) and bin(3)) or 
	           (not(bin(0)) and bin(1) and not(bin(2)) and not(bin(3))) or 
				  (bin(0) and bin(1) and not(bin(2)) and bin(3)) or 
				  (bin(0) and not(bin(1)) and bin(2) and bin(3));

    seg(1) <= (not(bin(0)) and bin(1) and not(bin(2)) and bin(3)) or
              (bin(1) and bin(2) and not(bin(3))) or
              (bin(0) and bin(2) and bin(3)) or
              (bin(0) and bin(1) and not(bin(3)));

    seg(2) <= (not(bin(0)) and not(bin(1)) and bin(2) and not(bin(3))) or
              (bin(0) and bin(1) and bin(2)) or
              (bin(0) and bin(1) and not(bin(3)));

    seg(3) <= (not(bin(0)) and bin(1) and not(bin(2)) and not(bin(3))) or
              (not(bin(0)) and not(bin(1)) and not(bin(2)) and bin(3)) or
              (bin(0) and not(bin(1)) and bin(2) and not(bin(3))) or
              (bin(1) and bin(2) and bin(3));
    
    seg(4) <= (not(bin(0)) and bin(3)) or
              (not(bin(0)) and bin(1) and not(bin(2))) or
              (not(bin(1)) and not(bin(2)) and bin(3));

    seg(5) <= (bin(0) and bin(1) and not(bin(2)) and bin(3)) or
              (not(bin(0)) and not(bin(1)) and bin(3)) or
              (not(bin(0)) and bin(2) and bin(3)) or
              (not(bin(0)) and not(bin(1)) and bin(2));

    seg(6) <= (not(bin(0)) and not(bin(1)) and not(bin(2))) or
              (not(bin(0)) and bin(1) and bin(2) and bin(3)) or
              (bin(0) and bin(1) and not(bin(2)) and not(bin(3)));
end logicFunc;
